@@ -17,46 +17,43 @@
 {-# LANGUAGE PatternGuards         #-}
 {-# LANGUAGE DefaultSignatures     #-}
 {-# LANGUAGE NamedFieldPuns        #-}
-{-# LANGUAGE CPP                   #-}
 
 module Data.SBV.BitVectors.Data
  ( SBool, SWord8, SWord16, SWord32, SWord64
  , SInt8, SInt16, SInt32, SInt64, SInteger, SReal, SFloat, SDouble
- , nan, infinity, sNaN, sInfinity, RoundingMode(..), SRoundingMode, smtLibSquareRoot, smtLibFusedMA
+ , nan, infinity, sNaN, sInfinity, RoundingMode(..), SRoundingMode
  , sRoundNearestTiesToEven, sRoundNearestTiesToAway, sRoundTowardPositive, sRoundTowardNegative, sRoundTowardZero
+ , sRNE, sRNA, sRTP, sRTN, sRTZ
  , SymWord(..)
- , CW(..), CWVal(..), AlgReal(..), cwSameType, cwIsBit, cwToBool
+ , CW(..), CWVal(..), AlgReal(..), cwSameType, cwToBool
  , mkConstCW ,liftCW2, mapCW, mapCW2
  , SW(..), trueSW, falseSW, trueCW, falseCW, normCW
  , SVal(..)
  , SBV(..), NodeId(..), mkSymSBV
- , ArrayContext(..), ArrayInfo, SymArray(..), SFunArray(..), mkSFunArray, SArray(..), arrayUIKind
+ , ArrayContext(..), ArrayInfo, SymArray(..), SFunArray(..), mkSFunArray, SArray(..)
  , sbvToSW, sbvToSymSW, forceSWArg
  , SBVExpr(..), newExpr
  , cache, Cached, uncache, uncacheAI, HasKind(..)
- , Op(..), NamedSymVar, UnintKind(..), getTableIndex, SBVPgm(..), Symbolic, SExecutable(..), runSymbolic, runSymbolic', State, getPathCondition, extendPathCondition
+ , Op(..), FPOp(..), NamedSymVar, getTableIndex
+ , SBVPgm(..), Symbolic, SExecutable(..), runSymbolic, runSymbolic', State, getPathCondition, extendPathCondition
  , inProofMode, SBVRunMode(..), Kind(..), Outputtable(..), Result(..)
  , Logic(..), SMTLibLogic(..)
- , getTraceInfo, getConstraints, addConstraint
- , SBVType(..), newUninterpreted, unintFnUIKind, addAxiom
+ , addConstraint, internalVariable, internalConstraint, isCodeGenMode
+ , SBVType(..), newUninterpreted, addAxiom
  , Quantifier(..), needsExistentials
- , SMTLibPgm(..), SMTLibVersion(..)
+ , SMTLibPgm(..), SMTLibVersion(..), smtLibVersionExtension, smtLibReservedNames
  , SolverCapabilities(..)
  , extractSymbolicSimulationState
  , SMTScript(..), Solver(..), SMTSolver(..), SMTResult(..), SMTModel(..), SMTConfig(..), getSBranchRunConfig
  , declNewSArray, declNewSFunArray
  ) where
 
-#if __GLASGOW_HASKELL__ < 710
-import Control.Applicative  ((<$>))
-#endif
-
 import Control.DeepSeq      (NFData(..))
 import Control.Monad.Reader (ask)
 import Control.Monad.Trans  (liftIO)
 import Data.Int             (Int8, Int16, Int32, Int64)
 import Data.Word            (Word8, Word16, Word32, Word64)
-import Data.List            (intercalate, elemIndex)
+import Data.List            (elemIndex, intercalate)
 import Data.Maybe           (fromMaybe)
 
 import qualified Data.Generics as G    (Data(..))
@@ -69,75 +66,10 @@ import Data.SBV.Utils.Lib
 import Data.SBV.BitVectors.Kind
 import Data.SBV.BitVectors.Concrete
 import Data.SBV.BitVectors.Symbolic
+import Data.SBV.SMT.SMTLibNames
 
--- | A class for capturing values that have a sign and a size (finite or infinite)
--- minimal complete definition: kindOf. This class can be automatically derived
--- for data-types that have a 'Data' instance; this is useful for creating uninterpreted
--- sorts.
-class HasKind a where
-  kindOf          :: a -> Kind
-  hasSign         :: a -> Bool
-  intSizeOf       :: a -> Int
-  isBoolean       :: a -> Bool
-  isBounded       :: a -> Bool   -- NB. This really means word/int; i.e., Real/Float will test False
-  isReal          :: a -> Bool
-  isFloat         :: a -> Bool
-  isDouble        :: a -> Bool
-  isInteger       :: a -> Bool
-  isUninterpreted :: a -> Bool
-  showType        :: a -> String
-  -- defaults
-  hasSign x = kindHasSign (kindOf x)
-  intSizeOf x = case kindOf x of
-                  KBool         -> error "SBV.HasKind.intSizeOf((S)Bool)"
-                  KBounded _ s  -> s
-                  KUnbounded    -> error "SBV.HasKind.intSizeOf((S)Integer)"
-                  KReal         -> error "SBV.HasKind.intSizeOf((S)Real)"
-                  KFloat        -> error "SBV.HasKind.intSizeOf((S)Float)"
-                  KDouble       -> error "SBV.HasKind.intSizeOf((S)Double)"
-                  KUserSort s _ -> error $ "SBV.HasKind.intSizeOf: Uninterpreted sort: " ++ s
-  isBoolean       x | KBool{}      <- kindOf x = True
-                    | True                     = False
-  isBounded       x | KBounded{}   <- kindOf x = True
-                    | True                     = False
-  isReal          x | KReal{}      <- kindOf x = True
-                    | True                     = False
-  isFloat         x | KFloat{}     <- kindOf x = True
-                    | True                     = False
-  isDouble        x | KDouble{}    <- kindOf x = True
-                    | True                     = False
-  isInteger       x | KUnbounded{} <- kindOf x = True
-                    | True                     = False
-  isUninterpreted x | KUserSort{}  <- kindOf x = True
-                    | True                     = False
-  showType = show . kindOf
-
-  -- default signature for uninterpreted/enumerated kinds
-  default kindOf :: (Read a, G.Data a) => a -> Kind
-  kindOf = constructUKind
-
-instance HasKind Bool    where kindOf _ = KBool
-instance HasKind Int8    where kindOf _ = KBounded True  8
-instance HasKind Word8   where kindOf _ = KBounded False 8
-instance HasKind Int16   where kindOf _ = KBounded True  16
-instance HasKind Word16  where kindOf _ = KBounded False 16
-instance HasKind Int32   where kindOf _ = KBounded True  32
-instance HasKind Word32  where kindOf _ = KBounded False 32
-instance HasKind Int64   where kindOf _ = KBounded True  64
-instance HasKind Word64  where kindOf _ = KBounded False 64
-instance HasKind Integer where kindOf _ = KUnbounded
-instance HasKind AlgReal where kindOf _ = KReal
-instance HasKind Float   where kindOf _ = KFloat
-instance HasKind Double  where kindOf _ = KDouble
-
-instance HasKind Kind where
-  kindOf = id
-
-instance HasKind CW where
-  kindOf = cwKind
-
-instance HasKind SW where
-  kindOf (SW k _) = k
+import Prelude ()
+import Prelude.Compat
 
 -- | Get the current path condition
 getPathCondition :: State -> SBool
@@ -213,9 +145,6 @@ sInfinity = literal infinity
 -- | 'RoundingMode' can be used symbolically
 instance SymWord RoundingMode
 
--- | 'RoundingMode' kind
-instance HasKind RoundingMode
-
 -- | The symbolic variant of 'RoundingMode'
 type SRoundingMode = SBV RoundingMode
 
@@ -239,6 +168,26 @@ sRoundTowardNegative = literal RoundTowardNegative
 sRoundTowardZero :: SRoundingMode
 sRoundTowardZero = literal RoundTowardZero
 
+-- | Alias for 'sRoundNearestTiesToEven'
+sRNE :: SRoundingMode
+sRNE = sRoundNearestTiesToEven
+
+-- | Alias for 'sRoundNearestTiesToAway'
+sRNA :: SRoundingMode
+sRNA = sRoundNearestTiesToAway
+
+-- | Alias for 'sRoundTowardPositive'
+sRTP :: SRoundingMode
+sRTP = sRoundTowardPositive
+
+-- | Alias for 'sRoundTowardNegative'
+sRTN :: SRoundingMode
+sRTN = sRoundTowardNegative
+
+-- | Alias for 'sRoundTowardZero'
+sRTZ :: SRoundingMode
+sRTZ = sRoundTowardZero
+
 -- Not particularly "desirable", but will do if needed
 instance Show (SBV a) where
   show (SBV sv) = show sv
@@ -377,8 +326,8 @@ class (HasKind a, Ord a) => SymWord a where
     | True                  = False
 
   default literal :: Show a => a -> SBV a
-  literal x = let k@(KUserSort  _ (conts, _)) = kindOf x
-                  sx                          = show x
+  literal x = let k@(KUserSort  _ conts) = kindOf x
+                  sx                     = show x
                   mbIdx = case conts of
                             Right xs -> sx `elemIndex` xs
                             _        -> Nothing
@@ -445,7 +394,7 @@ class SymArray array where
 newtype SArray a b = SArray { unSArray :: SArr }
 
 instance (HasKind a, HasKind b) => Show (SArray a b) where
-  show (SArray{}) = "SArray<" ++ showType (undefined :: a) ++ ":" ++ showType (undefined :: b) ++ ">"
+  show SArray{} = "SArray<" ++ showType (undefined :: a) ++ ":" ++ showType (undefined :: b) ++ ">"
 
 instance SymArray SArray where
   newArray_                                      = declNewSArray (\t -> "array_" ++ show t)
