@@ -27,7 +27,7 @@ abc = SMTSolver {
          , engine       = standardEngine "SBV_ABC" "SBV_ABC_OPTIONS" addTimeOut standardModel
          , capabilities = SolverCapabilities {
                                 capSolverName              = "ABC"
-                              , mbDefaultLogic             = Nothing
+                              , mbDefaultLogic             = const Nothing
                               , supportsMacros             = True
                               , supportsProduceModels      = True
                               , supportsQuantifiers        = False
