@@ -12,7 +12,6 @@
 module TestSuite.Basics.IteTest(testSuite)  where
 
 import Data.SBV
-import Data.SBV.Internals
 
 import SBVTest
 
@@ -34,8 +33,5 @@ testSuite = mkTestSuite $ \goldCheck -> test [
  , "ite-4"  ~: assert =<< isThm (chk1 iteLazy)
  , "ite-5"  ~: assert =<< isThm (chk2 iteLazy)
  , "ite-6"  ~: assert =<< isThm (chk3 iteLazy)
- , "ite-7"  ~: assert =<< isThm (chk1 sBranch)
- , "ite-8"  ~: assert =<< isThm (chk2 sBranch)
- , "ite-9"  ~: assert =<< isThm (chk3 sBranch)
  ]
- where rs f = runSymbolic (True, Nothing) $ forAll ["x"] f
+ where rs f = runSAT $ forAll ["x"] f
