@@ -9,7 +9,9 @@
 -- Test suite for basic axioms and uninterpreted functions
 -----------------------------------------------------------------------------
 
+{-# LANGUAGE DeriveAnyClass     #-}
 {-# LANGUAGE DeriveDataTypeable #-}
+
 module TestSuite.Uninterpreted.Axioms(testSuite) where
 
 import Data.SBV
@@ -23,9 +25,7 @@ testSuite = mkTestSuite $ \_ -> test [
  ]
 
 -- Example provided by Thomas DuBuisson:
-data Bitstring = Bitstring () deriving (Eq, Ord, Data, Typeable, Read, Show)
-instance SymWord Bitstring
-instance HasKind Bitstring
+data Bitstring = Bitstring () deriving (Eq, Ord, Show, Read, Data, SymWord, HasKind)
 type SBitstring = SBV Bitstring
 
 a :: SBitstring -> SBool
